#include "StdAfx.h"
#include "object.h"
#include <fstream>

using namespace std;

object::object(string objectfile)
{
	fill();
	load_objectfile(objectfile);
}

object::object(const object &from_blueprint, int posx, int posy)
{
	fill();
	pic = from_blueprint.get_pic();
	destroy_graphics = false;
	pos[0] = posx;
	pos[1] = posy;
}

object::~object()
{
	if(destroy_graphics == true && pic)
		delete pic;
}

void object::fill()
{
	pos[0] = 0;
	pos[1] = 0;
	pic = 0;
	destroy_graphics = false;
}

void object::write_objectfile(string to, string picpath)
{
	ofstream file(to.c_str(), ios::out | ios::trunc | ios::binary);
	if(!file)
	{
		error::fatalError("Fatal error: Could not open \"" + picpath + "\" for write access.", false);
		file.close();
		return;
	}
	
	file << picpath;
	file.close();
}

void object::load_objectfile(string from)
{
	ifstream file(from.c_str(), ios::in | ios::binary);
	if(!file)
	{
		error::fatalError("Fatal error: Could not read \"" + from + "\"", true);
		file.close();
		return;	
	}
	
	string picpath;
	getline(file, picpath, '\n');
	pic = new asciibmp(picpath);
	destroy_graphics = true;
	
	file.close();
}

bool collides(int left1, int top1, int right1, int bottom1, int left2, int top2, int right2, int bottom2);

bool object::collides(const object *with)
{
	return ::collides(pos[0], pos[1], pos[0] + get_pic()->getW(), pos[1] + get_pic()->getH(),
		with->get_posx(), with->get_posy(), with->get_posx() + with->get_pic()->getW(), with->get_posy() + with->get_pic()->getH());
}

bool collides(int left1, int top1, int right1, int bottom1, int left2, int top2, int right2, int bottom2)
{
	bool collides[4] = { false, false, false, false };

	//Left side colliding
	if(left1 <= right2 && left1 >= left2)
	{
		collides[0] = true;
	}
	//Right side colliding
	if(right1 >= left2 && right1 <= right2)
	{
		collides[1] = true;
	}
	//Top side colliding
	if(top1 <= bottom2 && top1 >= top2)
	{
		collides[2] = true;
	}
	//Bottom side colliding
	if(bottom1 >= top2 && bottom1 <= bottom2)
	{
		collides[3] = true;
	}

	if(!(collides[0] || collides[1] || collides[2] || collides[3]))
		return false;

	//Check total collision for all 4 corners
	if(collides[0] && collides[2])
		return true;
	if(collides[0] && collides[3])
		return true;
	if(collides[1] && collides[2])
		return true;
	if(collides[1] && collides[3])
		return true;

	if(bottom1 - top1 > bottom2 - top2)
	{
		if((collides[0] || collides[1]) && top1 < top2 && bottom1 > bottom2)
			return true;
	}
	if(right1 - left1 > right2 - left2)
	{
		if((collides[2] || collides[3]) && left1 < left2 && right1 > right2)
			return true;
	}

	return false;
}
